@@ -2,20 +2,25 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2020, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
-__version__ = '1.0.0'
+from ufit.version import get_version
 
+__version__ = get_version()
 
-def debug(str):
-    if __debug__:
-        print str
 
 class UFitError(Exception):
     pass
 
+import matplotlib
+try:
+    import ufit.qt
+    matplotlib.use('Qt5Agg')
+except:
+    pass
+
 from ufit.result import *
 from ufit.param import *
 from ufit.data import *
