@@ -2,13 +2,13 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
 """Backend modules for different fitting packages/algorithms."""
 
-from ufit import debug, UFitError
+from ufit import UFitError
 
 __all__ = ['set_backend', 'backend']
 
@@ -54,4 +54,4 @@ def set_backend(which):
     backend = globals()[which]
     if backend is None:
         raise UFitError('Backend %r is not available' % which)
-    debug('ufit using %s backend' % backend.backend_name)
+    print('ufit using %s backend' % backend.backend_name)
