@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -10,14 +10,15 @@
 
 from __future__ import absolute_import
 
-from ufit.param import prepare_params, update_params
+from lmfit import Parameters, minimize, report_fit
 
-from lmfit import Parameters, minimize
+from ufit.param import prepare_params, update_params
 
 __all__ = ['do_fit', 'backend_name']
 
 backend_name = 'lmfit'
 
+
 def do_fit(data, fcn, params, add_kw):
 
     # lmfit can handle expression-based parameters itself, but that is
@@ -37,18 +38,24 @@ def do_fit(data, fcn, params, add_kw):
         update_params(dependent, meta, pd)
         return (fcn(pd, x) - y) / dy
 
+    printReport = add_kw.pop('printReport', False)
+
     try:
         out = minimize(lmfitfcn, lmfparams, args=(data,), **add_kw)
-    except Exception, e:
+    except Exception as e:
+        print(str(e))
         return False, str(e), 0
 
-    pd = dict((pn, lmfparams[pn].value) for pn in varynames)
+    if printReport:
+        report_fit(out.params)
+
+    pd = dict((pn, out.params[pn].value) for pn in varynames)
     update_params(dependent, meta, pd)
     for p in params:
         p.value = pd[p.name]
         if p.name in lmfparams:
-            p.error = lmfparams[p.name].stderr
-            p.correl = lmfparams[p.name].correl
+            p.error = out.params[p.name].stderr
+            p.correl = out.params[p.name].correl
         else:
             p.error = 0
             p.correl = {}
