@@ -2,13 +2,14 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
 """Backend using plain scipy leastsq."""
 
 from __future__ import absolute_import
+
 from numpy import sqrt, inf
 from scipy.optimize import leastsq
 
@@ -19,6 +20,7 @@ __all__ = ['do_fit', 'backend_name']
 
 backend_name = 'scipy'
 
+
 def do_fit(data, fcn, params, add_kw):
     x, y, dy = data.fit_columns
     meta = data.meta
@@ -34,12 +36,12 @@ def do_fit(data, fcn, params, add_kw):
     for p in varying:
         initpars.append(p.value)
         if (p.pmin is not None or p.pmax is not None) and not warned:
-            print 'Sorry, scipy backend cannot handle parameter bounds.'
+            print('Sorry, scipy backend cannot handle parameter bounds.')
             warned = True
 
     try:
         res = leastsq(leastsqfcn, initpars, args=(data,), full_output=1, **add_kw)
-    except Exception, e:
+    except Exception as e:
         return False, str(e), 0
 
     popt, pcov, infodict, errmsg, ier = res
@@ -56,7 +58,7 @@ def do_fit(data, fcn, params, add_kw):
     for i, p in enumerate(varying):
         pd[p.name] = popt[i]
         if pcov is not inf:
-            p.error = sqrt(pcov[i,i])
+            p.error = sqrt(pcov[i, i])
         else:
             p.error = 0
         p.correl = {}  # XXX
