@@ -2,18 +2,16 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
 """Backend using "unifit" leastsq algorithm."""
 
-from __future__ import absolute_import
-
 from copy import copy
 from time import time
 from numpy import sqrt, infty, ones, zeros, absolute, maximum, minimum, \
-     finfo, corrcoef, nonzero, diag, isnan, dot
+    finfo, corrcoef, nonzero, diag, isnan, dot
 from numpy.linalg import svd, pinv
 
 from ufit.param import prepare_params, update_params
@@ -23,6 +21,7 @@ __all__ = ['do_fit', 'backend_name']
 
 backend_name = 'unifit'
 
+
 def do_fit(data, fcn, params, add_kw):
     x, y, dy = data.fit_columns
     meta = data.meta
@@ -40,12 +39,12 @@ def do_fit(data, fcn, params, add_kw):
         initpars.append(p.value)
         initdp.append(p.delta or p.value/10. or 0.01)
         if (p.pmin is not None or p.pmax is not None) and not warned:
-            print 'Sorry, unifit backend cannot handle parameter bounds.'
+            print('Sorry, unifit backend cannot handle parameter bounds.')
             warned = True
     try:
         res = __leastsq((x, y, dy), leastsqfcn, initpars, initdp,
                         **add_kw)
-    except Exception, e:
+    except Exception as e:
         raise
         return False, str(e), 0
 
@@ -67,6 +66,7 @@ def do_fit(data, fcn, params, add_kw):
 class FitError(Exception):
     pass
 
+
 def __dfdp(x, f, p, dp, func):
     """
     Returns the partial derivatives of function 'func'.
@@ -76,7 +76,7 @@ def __dfdp(x, f, p, dp, func):
     """
 
     y = zeros((len(x), len(p)))
-    #pp = copy(p)
+    # pp = copy(p)
     dpp = copy(dp)
 
     for i in range(len(p)):
@@ -85,10 +85,11 @@ def __dfdp(x, f, p, dp, func):
             dpp[i] = 0.00001
         pp[i] = p[i] + dpp[i]
         t = func(x, pp)
-        y[:,i] = (t-f)/dpp[i]
+        y[:, i] = (t-f)/dpp[i]
     return y
 
-def __leastsq((x, y, wt), func, p, dp, niter=50, mode='print', eps=1e-3):
+
+def __leastsq(xyw, func, p, dp, niter=50, mode='print', eps=1e-3):
     """Levenberg-Marquardt nonlinear regression of func(x,p) to y(x).
 
              ---------------
@@ -113,7 +114,7 @@ def __leastsq((x, y, wt), func, p, dp, niter=50, mode='print', eps=1e-3):
     Bard, Nonlinear Parameter Estimation, Academic Press, 1974.
     Draper and Smith, Applied Regression Analysis, John Wiley and Sons, 1981.
     """
-    #processing of input arguments
+    (x, y, wt) = xyw
     options = zeros((len(p),2))            #desired fractional precision in parameter estimates.
     options[:,1] = ones(len(p)) * infty    #maximum fractional step change in parameter vector.
                         #not yet implemented standards are set (zero for precision, infty for max change)
@@ -240,13 +241,13 @@ def __leastsq((x, y, wt), func, p, dp, niter=50, mode='print', eps=1e-3):
 
         epsLlast = epsL
         if ss < finfo(float).eps:
-            print 'Sum of squares within machine precision'
+            print('Sum of squares within machine precision')
             break # machine precession
 
         aprec=absolute(pprec*pbest)
         if ((absolute(chg)).all() < aprec.all()) and ((absolute(chgprev)).all() < aprec.all()):
             kvg = True
-            print 'Parameter changes converged to specified precision'
+            print('Parameter changes converged to specified precision')
             break
         else:
             chgprev = chg
