@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -11,60 +11,64 @@ Utilities for clustering execution of a piece of code to multiple hosts using
 SSH transport.
 """
 
+from __future__ import print_function
+
 import sys
 import md5
-import Queue
 import threading
-import cPickle as pickle
 from os import path
 from time import time
 
 import paramiko
 
+from ufit.pycompat import queue, cPickle as pickle
+
 keyname = path.expanduser('~/.ufitcluster/key')
 clusterlist = []
 
+
 def init_cluster():
     try:
         fp = open(path.expanduser('~/.ufitcluster/hosts'))
-        for line in fp: # user@host
+        for line in fp:  # user@host
             if line.startswith('#'):
                 continue
             login, host = line.strip().split('@')
             clusterlist.append((login, host))
         fp.close()
-    except Exception, err:
-        print >>sys.stderr, 'Cannot read the list of cluster hosts:', err
-        print >>sys.stderr, 'Please create a file %s with one line for each ' \
-            'parallel process, for example:' % \
-            path.expanduser('~/.ufitcluster/hosts')
-        print >>sys.stderr
-        print >>sys.stderr, 'username@hostname1'
-        print >>sys.stderr, 'username@hostname1'
-        print >>sys.stderr, 'username@hostname1'
-        print >>sys.stderr, 'username@hostname1'
-        print >>sys.stderr, 'username@hostname2'
-        print >>sys.stderr, 'username@hostname2'
-        print >>sys.stderr
-        print >>sys.stderr, 'would run 4 parallel processes on hostname1 and ' \
-            '2 parallel processes on hostname2.'
-        print >>sys.stderr, 'For local operation only, use "localhost" as ' \
-            'hostname.'
-        print >>sys.stderr, 'The file %s must contain the SSH private key to ' \
-            'use for the connection, even for localhost.' % \
-            path.expanduser('~/.ufitcluster/key')
+    except Exception as err:
+        print('Cannot read the list of cluster hosts:', err, file=sys.stderr)
+        print('Please create a file %s with one line for each '
+              'parallel process, for example:' %
+              path.expanduser('~/.ufitcluster/hosts'), file=sys.stderr)
+        print(file=sys.stderr)
+        print('username@hostname1', file=sys.stderr)
+        print('username@hostname1', file=sys.stderr)
+        print('username@hostname1', file=sys.stderr)
+        print('username@hostname1', file=sys.stderr)
+        print('username@hostname2', file=sys.stderr)
+        print('username@hostname2', file=sys.stderr)
+        print(file=sys.stderr)
+        print('would run 4 parallel processes on hostname1 and '
+              '2 parallel processes on hostname2.', file=sys.stderr)
+        print('For local operation only, use "localhost" as '
+              'hostname.', file=sys.stderr)
+        print('The file %s must contain the SSH private key to '
+              'use for the connection, even for localhost.' %
+              path.expanduser('~/.ufitcluster/key'), file=sys.stderr)
 
 
 def client_runner(client, task_queue, result_queue):
     while True:
         code, funcname, jobnum, args = task_queue.get()
         if jobnum == -1:
-            print '[C] exiting runner for %s' % client._host
+            print('[C] exiting runner for %s' % client._host)
             return
         try:
             sid = md5.new(str(time()) + str(args)).hexdigest()
-            sys.stdout.write('.'); sys.stdout.flush()
-            #print '[C] starting job on %s: %s' % (client._host, sid)
+            sys.stdout.write('.')
+            sys.stdout.flush()
+            # print '[C] starting job on %s: %s' % (client._host, sid)
             code_footer = '''\nif __name__ == "__main__":
             import cPickle as pickle
             args = pickle.loads(%r)
@@ -80,20 +84,21 @@ def client_runner(client, task_queue, result_queue):
                 client.exec_command('python /tmp/ufit_cluster_%s.py; '
                                     'rm /tmp/ufit_cluster_%s.py' % (sid, sid))
             result = pickle.load(stdout)
-            #print '[C] done with job %s: %r' % (sid, result)
+            # print '[C] done with job %s: %r' % (sid, result)
             result_queue.put((jobnum, result))
-        except Exception, err:
-            print '[C] no result on %s, requeuing: %r' % (client._host, err)
+        except Exception as err:
+            print('[C] no result on %s, requeuing: %r' % (client._host, err))
             task_queue.put((code, funcname, jobnum, args))
             return
 
 
 clients = []
 runners = []
-task_queue = Queue.Queue()
-result_queue = Queue.Queue()
+task_queue = queue.Queue()
+result_queue = queue.Queue()
 cluster_setup = False
 
+
 def setup_cluster():
     global cluster_setup
     if cluster_setup:
@@ -105,13 +110,14 @@ def setup_cluster():
         client.connect(host, username=user, key_filename=keyname,
                        look_for_keys=False)
         clients.append(client)
-        print '[C] opened cluster connection to %s' % client._host
+        print('[C] opened cluster connection to %s' % client._host)
         runner = threading.Thread(target=client_runner,
-            args=(client, task_queue, result_queue))
+                                  args=(client, task_queue, result_queue))
         runners.append(runner)
         runner.start()
     cluster_setup = True
 
+
 def kill_cluster():
     for cl in clients:
         task_queue.put((None, None, -1, None))  # end!
@@ -138,6 +144,6 @@ init_cluster()
 
 if __name__ == '__main__':
     t1 = time()
-    print run_cluster('def foo(a): return a\n', 'foo', [('a',),])
+    print(run_cluster('def foo(a): return a\n', 'foo', [('a',)]))
     t2 = time()
-    print t2-t1
+    print(t2-t1)
