@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -10,9 +10,11 @@
 
 from ufit import UFitError
 from ufit.data import ill, nicos, nicos_old, simple, simple_csv, trisp, \
-    llb, cascade, taipan
+    llb, cascade, taipan, nist
 from ufit.data.loader import Loader
 from ufit.data.dataset import Dataset, ScanData, ImageData, DatasetList
+from ufit.plotting import mapping
+from ufit.pycompat import listitems
 
 data_formats_scan = {
     'ill': ill,
@@ -23,18 +25,19 @@ data_formats_scan = {
     'trisp': trisp,
     'llb': llb,
     'taipan': taipan,
+    'nist': nist,
 }
 
 data_formats_image = {
     'cascade': cascade,
 }
 
-data_formats = dict(kv for kv in (data_formats_scan.items() +
-                                  data_formats_image.items()))
+data_formats = dict(listitems(data_formats_scan) +
+                    listitems(data_formats_image))
 
 __all__ = ['Dataset', 'DatasetList', 'ScanData', 'ImageData', 'sets',
            'set_datatemplate', 'set_dataformat', 'read_data', 'as_data',
-           'read_numors']
+           'read_numors', 'do_mapping']
 
 
 # simplified interface for usage in noninteractive scripts
@@ -42,6 +45,7 @@ __all__ = ['Dataset', 'DatasetList', 'ScanData', 'ImageData', 'sets',
 global_loader = Loader()
 sets = global_loader.sets
 
+
 def set_datatemplate(template):
     """Set a new template for data file names.
 
@@ -59,6 +63,7 @@ def set_datatemplate(template):
     """
     global_loader.template = template
 
+
 def set_dataformat(format):
     """Set the input data format.
 
@@ -72,6 +77,7 @@ def set_dataformat(format):
     * ``'old nicos'`` - NICOS 1.0 data format
     * ``'trisp'`` - FRM-II TRISP data format
     * ``'taipan'`` - ANSTO Taipan data format
+    * ``'nist'`` - NIST data format
     * ``'simple'`` - simple whitespace-separated multi-column files
     * ``'simple comma-separated'`` - simple comma-separated multi-column files
     """
@@ -80,7 +86,8 @@ def set_dataformat(format):
                         % (format, ', '.join(data_formats)))
     global_loader.format = format
 
-def read_data(n, xcol='auto', ycol='auto', dycol=None, ncol=None, nscale=1):
+
+def read_data(n, xcol='auto', ycol='auto', dycol=None, ncol=None, nscale=1, filter=None):
     """Read a data file.  Returns a :class:`Dataset` object.
 
     :param xcol: X column name (or 1-based index)
@@ -91,21 +98,26 @@ def read_data(n, xcol='auto', ycol='auto', dycol=None, ncol=None, nscale=1):
         monitor column
     :param nscale: scale for the normalization column; the Y data is determined
         as ``y[i] = y_raw[i] / ncol[i] * nscale``
+    :param filter: dictionary of column names and filtered values; only datapoints
+        with given values will be part of the dataset; if None, then not applied.
     """
-    return global_loader.load(n, xcol, ycol, dycol, ncol, nscale)
+    return global_loader.load(n, xcol, ycol, dycol, ncol, nscale, filter)
+
 
 def as_data(x, y, dy, name=''):
     """Quickly construct a :class:`Dataset` object from three numpy arrays."""
     return Dataset.from_arrays(name or 'data', x, y, dy)
 
+
 def read_numors(nstring, binsize, xcol='auto', ycol='auto',
-                dycol=None, ncol=None, nscale=1):
+                dycol=None, ncol=None, nscale=1, floatmerge=True):
     """Read a number of data files.  Returns a list of :class:`Dataset`\s.
 
     :param nstring: A string that gives file numbers, with the operators given
         below.
     :param binsize: Bin size when files need to be merged according to
         *nstring*.
+    :param floatmerge: If to use float merging instead of binning. Default true.
 
     Other parameters as in :func:`read_data`.
 
@@ -125,4 +137,31 @@ def read_numors(nstring, binsize, xcol='auto', ycol='auto',
     * ``'10,11,12+13,14'`` loads four sets.
     """
     return global_loader.load_numors(nstring, binsize, xcol, ycol,
-                                     dycol, ncol, nscale)
+                                     dycol, ncol, nscale, floatmerge)
+
+
+def do_mapping(x, y, runs, minmax=None, mode=0, log=False, dots=True,
+               xscale=1, yscale=1, interpolate=100, usemask=True, figure=None,
+               clear=True, colors=None, axes=None, title=None):
+    """Create a 2D map from several datasets.
+
+    An example::
+
+        mapping("h", "E", datas)
+
+    :param minmax: tuple of minimum and maximum value for z-scale
+    :param mode: 0 = image, 1 = contour filled, 2 = contour lines
+    :param log: True if the z-scale should be logarithmic
+    :param dots: whether to show dots on position of datapoints
+    :param xscale, yscale: to define scale between x and y axis
+    :param interpolate: number of points to interpolate in between points
+    :param usemask: True if masked out points should be removed
+    :param figure: where to plot, default is current figure (``pl.gcf()``)
+    :param clear: whether to do ``figure.clf()`` before plotting
+    :param colors: when using mode > 1, optional list of colors for plotting
+    :param axes: axes where to plot, default is figure.gca()
+    :param title: title of the plot
+    """
+    return mapping(x, y, runs, minmax, mode, log, dots,
+                   xscale, yscale, interpolate, usemask, figure,
+                   clear, colors, axes, title)
