@@ -2,16 +2,19 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2020, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
 """Data loader object."""
 
+import io
+
 from numpy import ones, sqrt
 
 from ufit import UFitError
 from ufit.data.dataset import ScanData, ImageData, DataList, DatasetList
+from ufit.pycompat import iteritems, string_types, number_types
 
 
 class Loader(object):
@@ -23,26 +26,27 @@ class Loader(object):
     def _get_reader(self, filename, fobj):
         from ufit.data import data_formats, data_formats_image
         if self.format == 'auto':
-            for n, m in data_formats.iteritems():
-                # check 'simple' last
-                if n != 'simple' and m.check_data(fobj):
+            for n, m in iteritems(data_formats):
+                # check 'simple' formats last
+                if not n.startswith('simple') and m.check_data(fobj):
+                    return m, n in data_formats_image
+            for n, m in iteritems(data_formats):
+                if n.startswith('simple') and m.check_data(fobj):
                     return m, n in data_formats_image
-            if data_formats['simple'].check_data(fobj):
-                return data_formats['simple'], False
             raise UFitError('File %r has no recognized file format' % filename)
         return data_formats[self.format], self.format in data_formats_image
 
-    def _inner_load(self, n, xcol, ycol, dycol=None, ncol=None, nscale=1):
+    def _inner_load(self, n, xcol, ycol, dycol=None, ncol=None, nscale=1, filter=None):
         try:
             filename = self.template % n
         except TypeError:
             filename = self.template
-        fobj = open(filename, 'rb')
+        fobj = io.open(filename, 'rb')
         rdr, isimg = self._get_reader(filename, fobj)
         if isimg:
             return self._inner_load_image(rdr, filename, fobj, n, ncol, nscale)
         return self._inner_load_scan(rdr, filename, fobj, n, xcol, ycol, dycol,
-                                     ncol, nscale)
+                                     ncol, nscale, filter)
 
     def _inner_load_image(self, rdr, filename, fobj, n, ncol, nscale):
         arr, darr, meta = rdr.read_data(filename, fobj)
@@ -60,8 +64,14 @@ class Loader(object):
         return dset
 
     def _inner_load_scan(self, rdr, filename, fobj, n,
-                         xcol, ycol, dycol, ncol, nscale):
+                         xcol, ycol, dycol, ncol, nscale, filter):
         colnames, coldata, meta = rdr.read_data(filename, fobj)
+        if filter is not None:
+            for v, k in filter.items():
+                if v in colnames:
+                    coldata = coldata[coldata.T[colnames.index(v)] == k]
+                else:
+                    raise UFitError("Filtered column %s did not exists" % v)
         colguess = rdr.guess_cols(colnames, coldata, meta)
         if 'filenumber' not in meta:
             meta['filenumber'] = n
@@ -71,7 +81,7 @@ class Loader(object):
         datarr = ones((len(coldata), 4))
 
         def colindex(col):
-            if isinstance(col, str):
+            if isinstance(col, string_types):
                 try:
                     return colnames.index(col)
                 except ValueError:
@@ -87,27 +97,27 @@ class Loader(object):
             use_hkl = True
         if xcol == 'auto':
             xcol = colguess[0]
-        datarr[:,0] = coldata[:,colindex(xcol)]
+        datarr[:, 0] = coldata[:, colindex(xcol)]
         if ycol == 'auto':
             ycol = colguess[1]
-        datarr[:,1] = coldata[:,colindex(ycol)]
+        datarr[:, 1] = coldata[:, colindex(ycol)]
         if dycol == 'auto':
             dycol = colguess[2]
         if dycol is not None:
-            datarr[:,2] = coldata[:,colindex(dycol)]
+            datarr[:, 2] = coldata[:, colindex(dycol)]
         else:
-            datarr[:,2] = sqrt(datarr[:,1])
+            datarr[:, 2] = sqrt(datarr[:, 1])
         if ncol == 'auto':
             ncol = colguess[3]
         if ncol is not None:
-            datarr[:,3] = coldata[:,colindex(ncol)]
+            datarr[:, 3] = coldata[:, colindex(ncol)]
             if nscale == -1:
-                nscale = int(float('%.2g' % datarr[:,3].mean()))
+                nscale = int(float('%.2g' % datarr[:, 3].mean()))
 
         def colname(col):
             if col is None:
                 return None
-            elif isinstance(col, str):
+            elif isinstance(col, string_types):
                 return col
             return colnames[col - 1]   # 1-based indices
         if use_hkl:
@@ -119,10 +129,10 @@ class Loader(object):
         self.sets[n] = dset
         return dset
 
-    def load(self, n, xcol, ycol, dycol=None, ncol=None, nscale=1):
+    def load(self, n, xcol, ycol, dycol=None, ncol=None, nscale=1, filter=None):
         try:
-            return self._inner_load(n, xcol, ycol, dycol, ncol, nscale)
-        except Exception, e:
+            return self._inner_load(n, xcol, ycol, dycol, ncol, nscale, filter)
+        except Exception as e:
             raise UFitError('Could not load data file %d: %s' % (n, e))
 
     def guess_cols(self, n):
@@ -130,7 +140,7 @@ class Loader(object):
             filename = self.template % n
         except TypeError:
             filename = self.template
-        fobj = open(filename, 'rb')
+        fobj = io.open(filename, 'rb')
         rdr, isimg = self._get_reader(filename, fobj)
         if isimg:
             arr, darr, meta = rdr.read_data(filename, fobj)
@@ -140,14 +150,14 @@ class Loader(object):
             else:
                 nmon = 1
             xguess, yguess, dyguess = None, None, None
-            colnames = meta.keys()
+            colnames = list(meta)
         else:
             colnames, coldata, meta = rdr.read_data(filename, fobj)
             xguess, yguess, dyguess, mguess = rdr.guess_cols(colnames, coldata, meta)
             if mguess is not None:
                 # use average monitor counts for normalization, but
                 # round to 2 significant digits
-                moncol = coldata[:,colnames.index(mguess)]
+                moncol = coldata[:, colnames.index(mguess)]
                 nmon = int(float('%.2g' % moncol.mean()))
             else:
                 nmon = 0
@@ -158,7 +168,7 @@ class Loader(object):
         return colnames, xguess, yguess, dyguess, mguess, nmon
 
     def load_numors(self, nstring, binsize, xcol, ycol, dycol=None,
-                    ncol=None, nscale=1, floatmerge=False):
+                    ncol=None, nscale=1, floatmerge=True, filter=None):
         """Load a number of data files and merge them according to numor
         list operations:
 
@@ -167,8 +177,9 @@ class Loader(object):
         * ``+`` - merge single files
         * ``>`` - merge sequential files
         """
-        if not isinstance(binsize, (int, float)):
+        if not isinstance(binsize, number_types):
             raise UFitError('binsize argument must be a number')
+
         def toint(a):
             try:
                 return int(a)
@@ -182,7 +193,7 @@ class Loader(object):
             if '-' in part1:
                 a, b = map(toint, part1.split('-'))
                 datasets.extend(
-                    self.load(n, xcol, ycol, dycol, ncol, nscale).merge(
+                    self.load(n, xcol, ycol, dycol, ncol, nscale, filter).merge(
                         binsize, floatmerge=floatmerge) for n in range(a, b+1))
             else:
                 parts2 = part1.split('+')
@@ -190,14 +201,14 @@ class Loader(object):
                 for part2 in parts2:
                     if '>' in part2:
                         a, b = map(toint, part2.split('>'))
-                        ds = [self.load(n, xcol, ycol, dycol, ncol, nscale)
+                        ds = [self.load(n, xcol, ycol, dycol, ncol, nscale, filter)
                               for n in range(a, b+1)]
                         inner.append(ds[0].merge(binsize, *ds[1:],
                                                  floatmerge=floatmerge))
                     else:
                         inner.append(
                             self.load(toint(part2), xcol, ycol, dycol,
-                                      ncol, nscale))
+                                      ncol, nscale, filter))
                 datasets.append(inner[0].merge(binsize, *inner[1:],
                                                floatmerge=floatmerge))
         return DatasetList(datasets)
