@@ -2,26 +2,35 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
 """Data merging routines."""
 
-from numpy import arange, ones, zeros, sqrt, array
+from numpy import arange, ones, zeros, sqrt, array, append, reshape
 
 from ufit import UFitError
 
 
-def rebin(data, binsize):
-    """Simple rebinning of (x, y, dy, n) data."""
-    # XXX make it work with hkle 4-d x data
+def rebin(data, binsize, meta={}):
+    """Simple rebinning of (x, y, dy, n) data and col_ meta."""
 
     if binsize == 0:
         # no merging, just concatenate
         return data
 
     x, y, dy, n = data.T
+    # copy meta
+    new_meta = meta.copy()
+    # identify columns
+    metanames = []
+    if meta != []:
+        for col in meta:
+            if not col.startswith('col_'):
+                continue
+            # add column to data
+            metanames.append(col)
 
     # calculate new x values
 
@@ -31,9 +40,9 @@ def rebin(data, binsize):
                    binsize) + halfbinsize
     nbins = len(stops)
 
-    # newarray will be the new x, y, dy, n array
-    newarray = zeros((nbins, 4))
-    newarray[:,0] = stops
+    # newarray will be the new x, y, dy, n and meta columns array
+    newarray = zeros((nbins, 4 + len(metanames)))
+    newarray[:, 0] = stops
 
     # this will keep track which data values we already used
     data_unused = ones(len(x), bool)
@@ -42,7 +51,7 @@ def rebin(data, binsize):
     # left out
     new_used = ones(nbins, bool)
 
-    for i in xrange(nbins):
+    for i in range(nbins):
         stop = newarray[i, 0]
         # get indices of all data points with x values lying below stop
         indices = x <= (stop + halfbinsize)
@@ -53,9 +62,10 @@ def rebin(data, binsize):
             newarray[i, 2] += sqrt((dy[indices]**2).sum())
             newarray[i, 3] += n[indices].sum()
             data_unused[indices] = False
+            for j, m in enumerate(metanames):
+                newarray[i, 4 + j] += meta[m][indices].sum() / sum(indices)
         else:
             new_used[i] = False
-
     # are there any data points left unused?
     if data_unused.any():
         raise UFitError('Merging data failed (data left over), check merging '
@@ -64,38 +74,62 @@ def rebin(data, binsize):
     # remove any stops without monitor data
     newarray = newarray[new_used]
 
-    # return array
-    return newarray
+    # extract merged meta information
+    for i, m in enumerate(metanames):
+        new_meta[m] = array(newarray[:,4 + i])
+    # return arrays
+    return array(newarray[:,:4]), new_meta
+
 
 def mergeList(tomerge):
     tomerge = array(tomerge)
     merged = tomerge.sum(axis = 0)
     merged[2] = sqrt((tomerge**2).sum(axis = 0)[2])
     merged[0] = merged[0] / len(tomerge)
-    return merged;
+    # average meta
+    merged[4:] = [x / len(tomerge) for x in merged[4:]]
+    return merged
 
-def floatmerge(data, binsize):
+
+def floatmerge(data, binsize, meta = []):
     """Merging data based on floating window."""
 
     if binsize == 0:
         # no merging, just concatenate
-        return data
+        return data, meta
 
     # sort data
-    data = data[data[:,0].argsort()]
+    sortorder = data[:, 0].argsort()
+    data = data[sortorder]
+    # sort meta and empty new meta columns
+    new_meta = meta.copy()
+    metanames = []
+    if meta:
+        for col in meta:
+            if not col.startswith('col_'):
+                continue
+            # add column to data
+            metanames.append(col)
+            data = append(data, reshape(meta.get(col, [])[sortorder], (-1, 1)),
+                          axis=1)
+
+    lastvals = []
+    tomerge = []
+    newlist = []
 
-    lastvals = None
-    tomerge =  []
-    newlist =  []
     for vals in data:
-        if lastvals != None:
+        if lastvals != []:  # no "if lastvals", could be an array
             if vals[0] > lastvals[0] + binsize:
-                #merge points in list:
+                # merge points in list:
                 newlist.append(mergeList(tomerge))
                 tomerge = []
         tomerge.append(vals)
         lastvals = vals
     newlist.append(mergeList(tomerge))
 
-    # return array
-    return array(newlist)
+    newlist = array(newlist)
+    # extract merged meta information
+    for i, m in enumerate(metanames):
+        new_meta[m] = array(newlist[:, 4 + i])
+    # return arrays
+    return array(newlist[:, :4]), new_meta
