@@ -2,13 +2,15 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
 """Load routine for NICOS2 data."""
 
+import io
 import time
+
 from numpy import array, loadtxt
 
 from ufit import UFitError
@@ -17,17 +19,21 @@ from ufit import UFitError
 def check_data(fp):
     dtline = fp.readline()
     fp.seek(0, 0)
-    return dtline.startswith('### NICOS data file')
+    return dtline.startswith(b'### NICOS data file')
 
 
-def _hkle_index(colnames):
+def _hkle_index(colnames, meta):
     # find the index of the "h" column or -1 if it's not a qscan
-    # NOTE: don't support "h" in a position other than 0 since this would
-    # recognize all scans as Q scans even if the HKL device is just added
-    # for monitoring purposes.
-    #if 'h' in colnames:
+    if meta.get('old_qscan') and 'h' in colnames:
+        qhindex = colnames.index('h')
+        if qhindex < len(colnames) - 3 and colnames[qhindex+3] == 'E':
+            return qhindex
+
+    # NOTE: for new NICOS scans don't support "h" in a position other
+    # than 0 since this would recognize all scans as Q scans even if the
+    # HKL device is just added for monitoring purposes.
     if colnames and colnames[0] == 'h':
-        qhindex = 0  # colnames.index('h')
+        qhindex = 0
         if qhindex < len(colnames) - 3 and colnames[qhindex+3] == 'E':
             return qhindex
     return -1
@@ -35,7 +41,7 @@ def _hkle_index(colnames):
 
 def guess_cols(colnames, coldata, meta):
     xg, yg, mg = None, None, None
-    qhindex = _hkle_index(colnames)
+    qhindex = _hkle_index(colnames, meta)
     if qhindex > -1:
         deviations = array([cs.max() - cs.min()
                             for cs in coldata.T[qhindex:qhindex+4]])
@@ -46,12 +52,12 @@ def guess_cols(colnames, coldata, meta):
     maxcts = 0
     for i, colname in enumerate(colnames):
         if colname.startswith('mon'):
-            if coldata[:,i].sum() > maxmon:
-                maxmon = coldata[:,i].sum()
+            if coldata[:, i].sum() > maxmon:
+                maxmon = coldata[:, i].sum()
                 mg = colname
         if colname.startswith(('det', 'ctr', 'psd.roi')):
-            if coldata[:,i].sum() > maxcts:
-                maxcts = coldata[:,i].sum()
+            if coldata[:, i].sum() > maxcts:
+                maxcts = coldata[:, i].sum()
                 yg = colname
     if yg is None:
         yg = colnames[1]
@@ -59,6 +65,7 @@ def guess_cols(colnames, coldata, meta):
 
 
 def read_data(filename, fp):
+    fp = io.TextIOWrapper(fp, 'ascii', 'ignore')
     meta = {}
     dtline = fp.readline()
     if not dtline.startswith('### NICOS data file'):
@@ -70,7 +77,6 @@ def read_data(filename, fp):
     meta['created'] = ctime
     remark = ''
     for line in iter(fp.readline, ''):
-        line = line.decode('ascii', 'ignore').encode('ascii')
         if line.startswith('### Scan data'):
             break
         if line.startswith('# '):
@@ -126,12 +132,12 @@ def _nicos_common_load(fp, colnames, colunits, meta, comments):
                   if colnames[i] != ';')
     colnames = [name for name in colnames if name != ';']
     colunits = [unit for unit in colunits if unit != ';']
-    usecols = cvdict.keys()
+    usecols = list(cvdict)
     coldata = loadtxt(fp, converters=cvdict, usecols=usecols, ndmin=2,
                       comments=comments)
     if not coldata.size:
         raise UFitError('empty data file')
-    cols = dict((name, coldata[:,i]) for (i, name) in enumerate(colnames))
+    cols = dict((name, coldata[:, i]) for (i, name) in enumerate(colnames))
     meta['environment'] = []
     for col in cols:
         meta[col] = cols[col].mean()
@@ -141,9 +147,9 @@ def _nicos_common_load(fp, colnames, colunits, meta, comments):
             break
     if 'B' in cols:
         meta['environment'].append('B = %.3f K' % meta['B'])
-    qhindex = _hkle_index(colnames)
+    qhindex = _hkle_index(colnames, meta)
     if qhindex > -1:
-        meta['hkle'] = coldata[:,qhindex:qhindex+4]
+        meta['hkle'] = coldata[:, qhindex:qhindex+4]
         deviations = array([cs.max() - cs.min()
                             for cs in coldata.T[qhindex:qhindex+4]])
         xg = colnames[qhindex + deviations.argmax()]
