@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2020, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -11,9 +11,8 @@
 import os
 from os import path
 
-from PyQt4.QtCore import pyqtSignature as qtsig, QByteArray
-from PyQt4.QtGui import QMainWindow, QApplication, QListWidgetItem, \
-    QVBoxLayout, QFileDialog
+from ufit.qt import pyqtSlot, QByteArray, QMainWindow, QApplication, \
+    QListWidgetItem, QVBoxLayout, QFileDialog
 
 from ufit.data import Loader
 from ufit.utils import extract_template
@@ -24,15 +23,39 @@ from ufit.gui.session import session
 from ufit.gui.scanitem import ScanDataItem
 
 
+class Unavailable(object):
+    def __format__(self, fmt):
+        # Accepts all format strings (important for e.g. ".2f")
+        return '???'
+
+
+class FormatWrapper(object):
+    """Wraps a dataset for the purposes of format() not raising exceptions."""
+
+    def __init__(self, dataset):
+        self.__dataset = dataset
+
+    def __getattr__(self, attr, default=Unavailable()):
+        val = getattr(self.__dataset, attr, default)
+        if isinstance(val, list):
+            return ', '.join(map(str, val))
+        return val
+
+    def __getitem__(self, attr):
+        return getattr(self, attr)
+
+
 class BrowseWindow(QMainWindow):
     def __init__(self, parent):
         QMainWindow.__init__(self, parent)
         loadUi(self, 'browse.ui')
         self.logger = logger.getChild('browse')
 
+        self.dataloader = parent
         self.rootdir = ''
         self.loader = Loader()
         self._data = {}
+        self.yaxis = None
         self.canvas = MPLCanvas(self)
         self.canvas.plotter.lines = True
         self.toolbar = MPLToolbar(self.canvas, self)
@@ -51,28 +74,42 @@ class BrowseWindow(QMainWindow):
             self.restoreState(windowstate)
             splitstate = settings.value('splitstate', QByteArray())
             self.splitter.restoreState(splitstate)
-            #vsplitstate = settings.value('vsplitstate').toByteArray()
-            #self.vsplitter.restoreState(vsplitstate)
+            self.monScaleEdit.setText(settings.value('fixedmonval'))
 
-    @qtsig('')
+    @pyqtSlot()
     def on_loadBtn_clicked(self):
         datas = [self._data[item.type()] for item in self.dataList.selectedItems()]
         if not datas:
             return
         items = [ScanDataItem(data) for data in datas]
-        # XXX which group
         session.add_items(items)
 
-    @qtsig('')
+    @pyqtSlot()
+    def on_addNumBtn_clicked(self):
+        numors = [self._data[item.type()].meta['filenumber']
+                  for item in self.dataList.selectedItems()]
+        if not numors:
+            return
+        self.dataloader.add_numors(sorted(numors))
+
+    @pyqtSlot()
     def on_dirBtn_clicked(self):
         newdir = QFileDialog.getExistingDirectory(self, 'New directory',
                                                   self.rootdir)
         self.set_directory(path_to_str(newdir))
 
-    @qtsig('')
+    @pyqtSlot()
     def on_refreshBtn_clicked(self):
         self.set_directory(self.rootdir)
 
+    @pyqtSlot()
+    def on_monScaleBtn_clicked(self):
+        self.set_directory(self.rootdir)
+
+    @pyqtSlot()
+    def on_yAxisBtn_clicked(self):
+        self.set_directory(self.rootdir)
+
     def set_directory(self, root):
         self.setWindowTitle('ufit browser - %s' % root)
         self.canvas.axes.text(0.5, 0.5, 'Please wait, loading all data...',
@@ -89,15 +126,27 @@ class BrowseWindow(QMainWindow):
             try:
                 t, n = extract_template(fn)
                 self.loader.template = t
-                res = self.loader.load(n, 'auto', 'auto', 'auto', 'auto', -1)
-            except Exception, e:
+                fixed_yaxis = self.yAxisEdit.text()
+                yaxis = fixed_yaxis if (fixed_yaxis and self.useYAxis.isChecked()) else 'auto'
+                res = self.loader.load(n, 'auto', yaxis, 'auto', 'auto', -1)
+            except Exception as e:
                 self.logger.warning('While loading %r: %s' % (fn, e))
             else:
+                if self.useMonScale.isChecked():
+                    const = int(self.monScaleEdit.text())  # XXX check
+                    res.rescale(const)
+                if self.useFmtString.isChecked():
+                    try:
+                        scanLabel = self.fmtStringEdit.text().format(
+                            n, FormatWrapper(res))
+                    except Exception:
+                        scanLabel = '%s (%s) - %s | %s' % (
+                            n, res.xcol, res.title, ', '.join(res.environment))
+                else:
+                    scanLabel = '%s (%s) - %s | %s' % (
+                        n, res.xcol, res.title, ', '.join(res.environment))
                 self._data[n] = res
-                QListWidgetItem('%s (%s) - %s - %s' %
-                                (n, res.xcol, res.title,
-                                 ', '.join(res.environment)),
-                                self.dataList, n)
+                QListWidgetItem(scanLabel, self.dataList, n)
         self.canvas.axes.clear()
         self.canvas.draw()
 
@@ -121,5 +170,4 @@ class BrowseWindow(QMainWindow):
             settings.setValue('geometry', self.saveGeometry())
             settings.setValue('windowstate', self.saveState())
             settings.setValue('splitstate', self.splitter.saveState())
-            #settings.setValue('vsplitstate',
-            #                  QVariant(self.vsplitter.saveState()))
+            settings.setValue('fixedmonval', self.monScaleEdit.text())
