@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2020, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -10,15 +10,14 @@
 
 import sys
 from os import path
-from cStringIO import StringIO
 
-from PyQt4 import uic
-from PyQt4.QtCore import SIGNAL, QSize, QSettings, Qt, QRectF, QByteArray
-from PyQt4.QtGui import QLineEdit, QSizePolicy, QWidget, QIcon, QFileDialog, \
-    QMessageBox, QPrinter, QPrintDialog, QPrintPreviewWidget, QPainter, QDialog
-from PyQt4.QtSvg import QSvgRenderer
+from ufit.qt import uic, pyqtSignal, QSize, QSettings, Qt, QRectF, \
+    QByteArray, QLineEdit, QSizePolicy, QWidget, QIcon, QFileDialog, \
+    QMessageBox, QPrinter, QPrintDialog, QPrintPreviewWidget, QPainter, \
+    QDialog, QSvgRenderer
 
-from matplotlib.backends.backend_qt4agg import \
+import matplotlib.backends.qt_editor.figureoptions
+from matplotlib.backends.backend_qt5agg import \
     FigureCanvasQTAgg as FigureCanvas, NavigationToolbar2QT, FigureManagerQT
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.colors import LogNorm
@@ -28,8 +27,7 @@ try:
     from matplotlib.backend_bases import key_press_handler
 except ImportError:
     key_press_handler = None
-from matplotlib.backends import backend_qt4
-backend_qt4.figureoptions = None  # disable toolbar button that doesn't work
+from matplotlib.backends import backend_qt5
 pyplot.rc('font', family='sans-serif')
 pyplot.rc('font', **{'sans-serif': 'Sans Serif, Arial, Helvetica, '
                      'Lucida Grande, Bitstream Vera Sans'})
@@ -37,25 +35,40 @@ pyplot.rc('font', **{'sans-serif': 'Sans Serif, Arial, Helvetica, '
 from ufit.gui import logger
 from ufit.gui.session import session
 from ufit.plotting import DataPlotter
+from ufit.pycompat import BytesIO, text_type, PY2
+from ufit.gui.ploteditor import figure_edit
+
+# override figure editor with our extended version
+matplotlib.backends.qt_editor.figureoptions.figure_edit = figure_edit
 
 uipath = path.dirname(__file__)
 
+
 def loadUi(widget, uiname, subdir='ui'):
     uic.loadUi(path.join(uipath, subdir, uiname), widget)
 
+
 def path_to_str(qstring):
-    return unicode(qstring).encode(sys.getfilesystemencoding())
+    if PY2:
+        return qstring.encode(sys.getfilesystemencoding())
+    return qstring
+
 
 def str_to_path(string):
-    return string.decode(sys.getfilesystemencoding())
+    if not isinstance(string, text_type):
+        return string.decode(sys.getfilesystemencoding())
+    return string
 
 
 class MPLCanvas(FigureCanvas):
     """Ultimately, this is a QWidget (as well as a FigureCanvasAgg, etc.)."""
+
+    logzChanged = pyqtSignal()
+    replotRequest = pyqtSignal()
+
     def __init__(self, parent, width=10, height=6, dpi=72, maincanvas=False):
         fig = Figure(figsize=(width, height), dpi=dpi)
         fig.set_facecolor('white')
-        self.printer = None
         self.print_width = 0
         self.main = parent
         self.logz = False
@@ -64,8 +77,9 @@ class MPLCanvas(FigureCanvas):
         # make tight_layout do the right thing
         self.axes.set_xlabel('x')
         self.axes.set_ylabel('y')
-        self.axes.set_title('(data title)\n(info)', size='medium')
+        self.axes.set_title('(data title)\n(info)')
         FigureCanvas.__init__(self, fig)
+
         # create a figure manager so that we can use pylab commands on the
         # main viewport
         def make_active(event):
@@ -81,10 +95,8 @@ class MPLCanvas(FigureCanvas):
         self.mpl_connect('key_press_event', self.key_press)
         # These will not do anything in standalone mode, but do not hurt.
         if maincanvas:
-            self.connect(session, SIGNAL('propsRequested'),
-                         self.on_session_propsRequested)
-            self.connect(session, SIGNAL('propsUpdated'),
-                         self.on_session_propsUpdated)
+            session.propsRequested.connect(self.on_session_propsRequested)
+            session.propsUpdated.connect(self.on_session_propsUpdated)
 
     def on_session_propsRequested(self):
         session.props.canvas_logz = self.logz
@@ -92,7 +104,7 @@ class MPLCanvas(FigureCanvas):
     def on_session_propsUpdated(self):
         if 'canvas_logz' in session.props:
             self.logz = session.props.canvas_logz
-            self.emit(SIGNAL('logzChanged'))
+            self.logzChanged.emit()
 
     def key_press(self, event):
         if key_press_handler:
@@ -116,41 +128,48 @@ class MPLCanvas(FigureCanvas):
         QWidget.resizeEvent(self, event)
 
     def print_(self):
-        sio = StringIO()
+        sio = BytesIO()
         self.print_figure(sio, format='svg')
         svg = QSvgRenderer(QByteArray(sio.getvalue()))
         sz = svg.defaultSize()
         aspect = sz.width()/float(sz.height())
 
-        if self.printer is None:
-            self.printer = QPrinter(QPrinter.HighResolution)
-            self.printer.setOrientation(QPrinter.Landscape)
+        printer = QPrinter(QPrinter.HighResolution)
+        printer.setOrientation(QPrinter.Landscape)
 
         dlg = QDialog(self)
         loadUi(dlg, 'printpreview.ui')
         dlg.width.setValue(self.print_width or 500)
-        ppw = QPrintPreviewWidget(self.printer, dlg)
+        ppw = QPrintPreviewWidget(printer, dlg)
         dlg.layout().insertWidget(1, ppw)
+
         def render(printer):
             height = printer.height() * (dlg.width.value()/1000.)
             width = aspect * height
             painter = QPainter(printer)
             svg.render(painter, QRectF(0, 0, width, height))
+
         def sliderchanged(newval):
             ppw.updatePreview()
-        self.connect(ppw, SIGNAL('paintRequested(QPrinter *)'), render)
-        self.connect(dlg.width, SIGNAL('valueChanged(int)'), sliderchanged)
+
+        ppw.paintRequested.connect(render)
+        dlg.width.valueChanged.connect(sliderchanged)
         if dlg.exec_() != QDialog.Accepted:
             return
         self.print_width = dlg.width.value()
-        pdlg = QPrintDialog(self.printer, self)
+        pdlg = QPrintDialog(printer, self)
         if pdlg.exec_() != QDialog.Accepted:
             return
-        render(self.printer)
+        render(printer)
+
+    def ufit_replot(self):
+        self.replotRequest.emit()
 
 
 class MPLToolbar(NavigationToolbar2QT):
 
+    popoutRequested = pyqtSignal()
+
     icon_name_map = {
         'home.png':         'magnifier-zoom-fit.png',
         'back.png':         'arrow-180.png',
@@ -180,24 +199,25 @@ class MPLToolbar(NavigationToolbar2QT):
     toolitems.append(('Execute', 'Show Python console', 'pyconsole',
                       'exec_callback'))
 
-    def _init_toolbar(self):
-        NavigationToolbar2QT._init_toolbar(self)
+    def __init__(self, *args, **kwds):
+        NavigationToolbar2QT.__init__(self, *args, **kwds)
         self.locLabel.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
         self._actions['logx_callback'].setCheckable(True)
         self._actions['logy_callback'].setCheckable(True)
         self._actions['logz_callback'].setCheckable(True)
-        self.connect(self.canvas, SIGNAL('logzChanged'),
-                     self.on_canvas_logzChanged)
+        self.canvas.logzChanged.connect(self.on_canvas_logzChanged)
 
-    def _icon(self, name):
+    def _icon(self, name, color=None):
         if name in self.icon_name_map:
             return QIcon(':/' + self.icon_name_map[name])
         return QIcon()
 
     def home(self):
         # always unzoom completely
-        self._views.clear()
-        self._positions.clear()
+        if hasattr(self, '_views'):
+            self._views.clear()
+        if hasattr(self, '_positions'):
+            self._positions.clear()
         self.canvas.figure.gca().autoscale()
         self.canvas.draw()
         return NavigationToolbar2QT.home(self)
@@ -241,7 +261,7 @@ class MPLToolbar(NavigationToolbar2QT):
         self.canvas.print_()
 
     def popout_callback(self):
-        self.emit(SIGNAL('popoutRequested'))
+        self.popoutRequested.emit()
 
     def exec_callback(self):
         try:
@@ -250,7 +270,8 @@ class MPLToolbar(NavigationToolbar2QT):
             logger.exception('Qt console window cannot be opened without '
                              'IPython; import error was:')
             QMessageBox.information(self, 'ufit',
-                'Please install IPython with qtconsole to activate this function.')
+                                    'Please install IPython with qtconsole to '
+                                    'activate this function.')
             return
         w = ConsoleWindow(self)
         w.ipython.executeCommand('from ufit.lab import *')
@@ -269,17 +290,17 @@ class MPLToolbar(NavigationToolbar2QT):
         filters = []
         for name, exts in sorted_filetypes:
             if 'eps' in exts or 'emf' in exts or 'jpg' in exts or \
-                'pgf' in exts or 'raw' in exts:
+               'pgf' in exts or 'raw' in exts:
                 continue
             exts_list = " ".join(['*.%s' % ext for ext in exts])
             filter = '%s (%s)' % (name, exts_list)
             filters.append(filter)
         filters = ';;'.join(filters)
-        fname = QFileDialog.getSaveFileName(self, 'Choose a filename to save to',
-                                            start, filters)
+        fname, _ = QFileDialog.getSaveFileName(self, 'Choose a filename to save to',
+                                               start, filters)
         if fname:
             try:
-                self.canvas.print_figure(unicode(fname))
+                self.canvas.print_figure(text_type(fname))
             except Exception as e:
                 logger.exception('Error saving file')
                 QMessageBox.critical(self, 'Error saving file', str(e))
@@ -294,7 +315,7 @@ class SmallLineEdit(QLineEdit):
 class SettingGroup(object):
     def __init__(self, name):
         self.name = name
-        self.settings = QSettings()
+        self.settings = QSettings('ufit', 'gui')
 
     def __enter__(self):
         self.settings.beginGroup(self.name)
