@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2020, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -10,9 +10,8 @@
 
 from os import path
 
-from PyQt4.QtCore import pyqtSignature as qtsig, SIGNAL, Qt
-from PyQt4.QtGui import QWidget, QFileDialog, QDialogButtonBox, QMessageBox, \
-    QMainWindow, QSplitter, QApplication
+from ufit.qt import pyqtSignal, pyqtSlot, Qt, QWidget, QFileDialog, \
+    QDialogButtonBox, QMessageBox, QMainWindow, QSplitter, QApplication
 
 from ufit.data import data_formats, Loader, ImageData
 from ufit.utils import extract_template
@@ -25,6 +24,8 @@ from ufit.gui.session import session
 
 
 class DataLoader(QWidget):
+    closeRequest = pyqtSignal()
+    newDatas = pyqtSignal(object, object)
 
     def __init__(self, parent, plotter, standalone=False):
         QWidget.__init__(self, parent)
@@ -37,14 +38,10 @@ class DataLoader(QWidget):
         self.sgroup = SettingGroup('main')
 
         # These will not do anything in standalone mode, but do not hurt.
-        self.connect(session, SIGNAL('propsRequested'),
-                     self.on_session_propsRequested)
-        self.connect(session, SIGNAL('propsUpdated'),
-                     self.on_session_propsUpdated)
-        self.connect(session, SIGNAL('itemsUpdated'),
-                     self.on_session_itemsUpdated)
-        self.connect(session, SIGNAL('groupAdded'),
-                     self.on_session_itemsUpdated)
+        session.propsRequested.connect(self.on_session_propsRequested)
+        session.propsUpdated.connect(self.on_session_propsUpdated)
+        session.itemsUpdated.connect(self.on_session_itemsUpdated)
+        session.groupAdded.connect(self.on_session_itemsUpdated)
 
         with self.sgroup as settings:
             data_template_path = settings.value('last_data_template', '')
@@ -68,7 +65,7 @@ class DataLoader(QWidget):
         if 'template' in session.props:
             self.templateEdit.setText(session.props.template)
 
-    def on_session_itemsUpdated(self):
+    def on_session_itemsUpdated(self, _ignored=None):
         # list of groups may have changed
         self.groupBox.clear()
         for group in session.groups:
@@ -77,7 +74,7 @@ class DataLoader(QWidget):
     def on_buttonBox_clicked(self, button):
         role = self.buttonBox.buttonRole(button)
         if role == QDialogButtonBox.RejectRole:
-            self.emit(SIGNAL('closeRequest'))
+            self.closeRequest.emit()
         elif role == QDialogButtonBox.NoRole:  # "preview"
             self.open_data()
         else:  # "open"
@@ -86,7 +83,7 @@ class DataLoader(QWidget):
     def on_dataformatBox_currentIndexChanged(self, i):
         self.loader.format = str(self.dataformatBox.currentText())
 
-    @qtsig('')
+    @pyqtSlot()
     def on_numorHelpBtn_clicked(self):
         QMessageBox.information(self, 'Numor Help', '''\
 The numor string contains file numbers, with the following operators:
@@ -109,23 +106,51 @@ into one set, as well as files 23 and 24.
         bwin = BrowseWindow(self)
         bwin.show()
         QApplication.processEvents()
-        bwin.set_directory(directory)
+        try:
+            bwin.set_directory(directory)
+        except OSError:
+            pass
         bwin.activateWindow()
 
-    @qtsig('')
+    def add_numors(self, numors):
+        ranges = []
+        prev = -1
+        start = None
+        last = None
+        for num in numors:
+            if last is not None:
+                if num != last + 1:
+                    ranges.append((start, last))
+                    start = num
+            else:
+                start = num
+            last = num
+
+        ranges.append((start, last))
+
+        s = ''.join(',%s' % ('%s' % s if s == e else '%s-%s' % (s, e))
+                    for (s, e) in ranges)
+        prev = self.numorsEdit.text()
+        if prev:
+            self.numorsEdit.setText(prev + s)
+        else:
+            self.numorsEdit.setText(s[1:])
+        self.open_data()  # preview
+
+    @pyqtSlot()
     def on_browseBtn_clicked(self):
         templ = path_to_str(self.templateEdit.text())
         self.open_browser(path.dirname(templ))
 
-    @qtsig('')
+    @pyqtSlot()
     def on_settemplateBtn_clicked(self):
-        previous = path_to_str(self.templateEdit.text())
+        previous = self.templateEdit.text()
         if previous:
             startdir = path.dirname(previous)
         else:
             startdir = '.'
         fn = path_to_str(QFileDialog.getOpenFileName(
-            self, 'Choose a file', startdir, 'All files (*)'))
+            self, 'Choose a file', startdir, 'All files (*)')[0])
         if not fn:
             return
         dtempl, numor = extract_template(fn)
@@ -139,7 +164,7 @@ into one set, as well as files 23 and 24.
         try:
             cols, xguess, yguess, dyguess, mguess, nmon = \
                 self.loader.guess_cols(numor)
-        except Exception, e:
+        except Exception as e:
             if not silent:
                 self.logger.exception('Could not read column names')
                 QMessageBox.information(self, 'Error',
@@ -159,11 +184,16 @@ into one set, as well as files 23 and 24.
         self.moncolBox.addItem('auto')
         self.moncolBox.addItem('none')
         self.moncolBox.setCurrentIndex(0)
+        self.filtercolBox.clear()
+        self.filtercolBox.addItem('none')
+        self.filtercolBox.setCurrentIndex(0)
+
         for i, name in enumerate(cols):
             self.xcolBox.addItem(name)
             self.ycolBox.addItem(name)
             self.dycolBox.addItem(name)
             self.moncolBox.addItem(name)
+            self.filtercolBox.addItem(name)
         self.monscaleEdit.setText(str(nmon or 1))
         self.numorsEdit.setText(str(numor))
         self.open_data()
@@ -179,6 +209,7 @@ into one set, as well as files 23 and 24.
         ycol = str(self.ycolBox.currentText())
         dycol = str(self.dycolBox.currentText())
         mcol = str(self.moncolBox.currentText())
+        fcol = str(self.filtercolBox.currentText())
         if mcol == 'none':
             mcol = None
         if dycol == 'sqrt(Y)':
@@ -189,20 +220,28 @@ into one set, as well as files 23 and 24.
             QMessageBox.information(
                 self, 'Error', 'Monitor scale must be integer.')
             return
+        if fcol == 'none':
+            filter = None
+        else:
+            try:
+                val = float(self.filtervalEdit.text())
+            except ValueError:
+                val = bytes(self.filtervalEdit.text(), 'utf-8')
+            filter = {fcol: val}
         dtempl = path_to_str(self.templateEdit.text())
         self.loader.template = dtempl
         numors = str(self.numorsEdit.text())
         try:
             datas = self.loader.load_numors(
-                numors, prec, xcol, ycol, dycol, mcol, mscale, floatmerge)
-        except Exception, e:
+                numors, prec, xcol, ycol, dycol, mcol, mscale, floatmerge, filter)
+        except Exception as e:
             self.logger.exception('Error while loading data file')
             QMessageBox.information(self, 'Error', str(e))
             return
         self.last_data = datas
         if final:
-            self.emit(SIGNAL('newDatas'), datas, self.groupBox.currentText())
-            self.emit(SIGNAL('closeRequest'))
+            self.newDatas.emit(datas, self.groupBox.currentText())
+            self.closeRequest.emit()
         else:
             self.plot()
 
@@ -241,7 +280,7 @@ class DataLoaderMain(QMainWindow):
         self.dloader.groupBoxLbl.hide()
         self.dloader.groupBoxDesc.hide()
         self.dloader.initialize()
-        self.connect(self.dloader, SIGNAL('closeRequest'), self.close)
+        self.dloader.closeRequest.connect(self.close)
         layout.addWidget(self.dloader)
         self.setCentralWidget(layout)
         self.setWindowTitle('Data loading')
