@@ -2,20 +2,20 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
 """Data fitter panel."""
 
-from PyQt4.QtCore import SIGNAL, Qt
-from PyQt4.QtGui import QApplication, QWidget, QMainWindow, QGridLayout, \
-    QFrame, QLabel, QDialogButtonBox, QCheckBox, QMessageBox, QSplitter, \
-    QComboBox, QKeySequence, QIcon
+from ufit.qt import pyqtSignal, Qt, QApplication, QWidget, QMainWindow, \
+    QGridLayout, QFrame, QLabel, QDialogButtonBox, QCheckBox, QMessageBox, \
+    QSplitter, QComboBox, QKeySequence, QIcon
 
 from ufit.gui import logger
 from ufit.gui.common import loadUi, MPLCanvas, MPLToolbar, SmallLineEdit
 from ufit.gui.session import session
+from ufit.pycompat import iteritems, number_types
 
 
 def is_float(x):
@@ -27,6 +27,9 @@ def is_float(x):
 
 
 class Fitter(QWidget):
+    closeRequest = pyqtSignal()
+    pickRequest = pyqtSignal(object)
+    replotRequest = pyqtSignal(object)
 
     def __init__(self, parent, standalone=False, fit_kws={}):
         QWidget.__init__(self, parent)
@@ -72,13 +75,13 @@ class Fitter(QWidget):
             oldp_dict = dict((p.name, p) for p in old_model.params)
             self.restore_from_params(oldp_dict)
 
-        self.connect(session, SIGNAL('modelFitted'), self.on_modelFitted)
+        session.modelFitted.connect(self.on_modelFitted)
 
         if self.standalone:
             if fit:
                 self.do_fit()
             else:
-                self.emit(SIGNAL('replotRequest'), None)
+                self.replotRequest.emit(None)
 
     def create_param_controls(self):
         self.param_controls = {}
@@ -93,7 +96,7 @@ class Fitter(QWidget):
         self.original_params = {}
         combo_items = [''] + [par.name for par in self.model.params] + \
             ['data.' + m for m in sorted(self.data.meta)
-             if isinstance(self.data.meta[m], (int, long, float))]
+             if isinstance(self.data.meta[m], number_types)]
         for p in self.model.params:
             e0 = QLabel(p.name, self)
             e1 = SmallLineEdit('%.5g' % p.value, self)
@@ -116,20 +119,19 @@ class Fitter(QWidget):
                 layout.addWidget(ctl, i, j)
             i += 1
             self.original_params[p.name] = p.copy()
-            self.connect(e1, SIGNAL('returnPressed()'), self.do_plot)
-            self.connect(e4.lineEdit(), SIGNAL('returnPressed()'), self.do_plot)
-            self.connect(e5, SIGNAL('returnPressed()'), self.do_plot)
-            self.connect(e6, SIGNAL('returnPressed()'), self.do_plot)
-            self.connect(e3, SIGNAL('clicked(bool)'), self.update_enables)
-            self.connect(e4, SIGNAL('editTextChanged(const QString&)'),
-                         self.update_enables)
+            e1.returnPressed.connect(self.do_plot)
+            e4.lineEdit().returnPressed.connect(self.do_plot)
+            e5.returnPressed.connect(self.do_plot)
+            e6.returnPressed.connect(self.do_plot)
+            e3.clicked.connect(self.update_enables)
+            e4.editTextChanged.connect(self.update_enables)
         layout.setRowStretch(i+1, 1)
         self.param_frame.setLayout(layout)
         self.param_scroll.setWidget(self.param_frame)
         self.update_enables()
 
     def update_enables(self, *ignored):
-        for p, ctls in self.param_controls.iteritems():
+        for p, ctls in iteritems(self.param_controls):
             # if there is an expr...
             if ctls[4].currentText():
                 # disable value and minmax, check "fixed" and disable "fixed"
@@ -162,7 +164,7 @@ class Fitter(QWidget):
     def on_buttonBox_clicked(self, button):
         role = self.buttonBox.buttonRole(button)
         if role == QDialogButtonBox.RejectRole:
-            self.emit(SIGNAL('closeRequest'))
+            self.closeRequest.emit()
         elif role == QDialogButtonBox.ApplyRole:
             self.do_fit()
         elif role == QDialogButtonBox.ActionRole:
@@ -178,7 +180,7 @@ class Fitter(QWidget):
                 self.statusLabel.setText('Saved parameter values restored.')
 
     def update_from_controls(self):
-        for p, ctls in self.param_controls.iteritems():
+        for p, ctls in iteritems(self.param_controls):
             _, val, _, fx, expr, pmin, pmax, delta = ctls
             p.value = float(val.text()) if val.text() else 0
             if fx.checkState() == Qt.Checked:
@@ -229,16 +231,18 @@ class Fitter(QWidget):
             self._pick_finished()
         else:
             self.statusLabel.setText('%s: click on %s' %
-                (self.picking, self._pick_points[len(self._pick_values)]))
+                                     (self.picking,
+                                      self._pick_points[len(self._pick_values)]))
 
     def do_pick(self, *args):
         if self.picking:
             return
-        self.emit(SIGNAL('pickRequest'), self)
+        self.pickRequest.emit(self)
         self._pick_points = self.model.get_pick_points()
         self._pick_values = []
         self.picking = 'Guess'
         self.statusLabel.setText('Guess: click on %s' % self._pick_points[0])
+
         def callback():
             self.model.apply_pick(self._pick_values)
             for p in self.model.params:
@@ -251,7 +255,7 @@ class Fitter(QWidget):
 
     def do_plot(self, *ignored):
         self.update_from_controls()
-        self.emit(SIGNAL('replotRequest'), None)
+        self.replotRequest.emit(None)
 
     def do_fit(self):
         if self.picking:
@@ -264,22 +268,22 @@ class Fitter(QWidget):
         QApplication.processEvents()
         try:
             res = self.model.fit(self.data, **self.fit_kws)
-        except Exception, e:
+        except Exception as e:
             self.logger.exception('Error during fit')
             self.statusLabel.setText('Error during fit: %s' % e)
             return
         self.on_modelFitted(self.item, res)
 
-        self.emit(SIGNAL('replotRequest'), True)
+        self.replotRequest.emit(True)
         session.set_dirty()
 
     def on_modelFitted(self, item, res):
         if item is not self.item:
             return
 
-        self.statusLabel.setText((res.success and 'Converged. ' or 'Failed. ')
-                                 + res.message +
-                                 ' Reduced chi^2 = %.3g.' % res.chisqr)
+        self.statusLabel.setText(
+            (res.success and 'Converged. ' or 'Failed. ') + res.message +
+            ' Reduced chi^2 = %.3g.' % res.chisqr)
 
         for p in res.params:
             self.param_controls[p][1].setText('%.5g' % p.value)
@@ -299,8 +303,8 @@ class FitterMain(QMainWindow):
         self.fitter = Fitter(self, standalone=True, fit_kws=fit_kws)
         self.canvas.mpl_connect('button_press_event',
                                 self.fitter.on_canvas_pick)
-        self.connect(self.fitter, SIGNAL('closeRequest'), self.close)
-        self.connect(self.fitter, SIGNAL('replotRequest'), self.replot)
+        self.fitter.closeRequest.connect(self.close)
+        self.fitter.replotRequest.connect(self.replot)
         self.fitter.initialize(model, data, fit)
         layout.addWidget(self.fitter)
         self.setCentralWidget(layout)
