@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2020, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -10,11 +10,9 @@
 
 from os import path
 
-from PyQt4.QtCore import pyqtSignature as qtsig, Qt, SIGNAL, QModelIndex, \
-    QByteArray
-from PyQt4.QtGui import QMainWindow, QVBoxLayout, QMessageBox, QMenu, QIcon, \
-    QFileDialog, QDialog, QAction, QActionGroup, QInputDialog, QSplitter, \
-    QLabel, QCursor
+from ufit.qt import pyqtSlot, Qt, QModelIndex, QByteArray, \
+    QMainWindow, QVBoxLayout, QMessageBox, QMenu, QIcon, QFileDialog, \
+    QDialog, QAction, QActionGroup, QInputDialog, QSplitter, QLabel, QCursor
 
 from ufit import backends, __version__
 from ufit.data.dataset import ScanData, ImageData
@@ -29,6 +27,8 @@ from ufit.gui.annotations import AnnotationWindow
 from ufit.gui.scanitem import ScanDataPanel, ScanDataItem
 from ufit.gui.imageitem import ImageDataItem
 from ufit.gui.session import session, temp_session, SessionItem, ItemGroup
+from ufit.gui.coverage import ReciprocalViewer
+from ufit.pycompat import listitems
 
 max_recent_files = 6
 
@@ -46,47 +46,48 @@ class UFitMain(QMainWindow):
         self.sgroup = SettingGroup('main')
         loadUi(self, 'main.ui')
 
-        self.connect(self.menuRecent, SIGNAL('aboutToShow()'),
-                     self.on_menuRecent_aboutToShow)
+        self.menuRecent.aboutToShow.connect(self.on_menuRecent_aboutToShow)
         self.menuMoveToGroup = QMenu('Move selected to group', self)
         self.menuMoveToGroup.setIcon(QIcon(':/drawer-open.png'))
-        self.connect(self.menuMoveToGroup, SIGNAL('aboutToShow()'),
-                     self.on_menuMoveToGroup_aboutToShow)
+        self.menuMoveToGroup.aboutToShow.connect(
+            self.on_menuMoveToGroup_aboutToShow)
+        self.menuCopyToGroup = QMenu('Copy selected to group', self)
+        self.menuCopyToGroup.setIcon(QIcon(':/drawer-double-open.png'))
+        self.menuCopyToGroup.aboutToShow.connect(
+            self.on_menuCopyToGroup_aboutToShow)
         self.menuRemoveGroup = QMenu('Remove group', self)
         self.menuRemoveGroup.setIcon(QIcon(':/drawer--minus.png'))
-        self.connect(self.menuRemoveGroup, SIGNAL('aboutToShow()'),
-                     self.on_menuRemoveGroup_aboutToShow)
+        self.menuRemoveGroup.aboutToShow.connect(
+            self.on_menuRemoveGroup_aboutToShow)
         self.menuRenameGroup = QMenu('Rename group', self)
-        self.connect(self.menuRenameGroup, SIGNAL('aboutToShow()'),
-                     self.on_menuRenameGroup_aboutToShow)
+        self.menuRenameGroup.aboutToShow.connect(
+            self.on_menuRenameGroup_aboutToShow)
 
         # populate plot view
         layout2 = QVBoxLayout()
         layout2.setContentsMargins(0, 0, 0, 0)
         self.canvas = MPLCanvas(self, maincanvas=True)
+        self.canvas.replotRequest.connect(
+            lambda: self.current_panel.plot(True))
         self.canvas.mpl_connect('button_press_event', self.on_canvas_pick)
         self.canvas.mpl_connect('pick_event', self.on_canvas_pick)
         self.canvas.mpl_connect('button_release_event', self.on_canvas_click)
         self.toolbar = MPLToolbar(self.canvas, self)
         self.toolbar.setObjectName('mainplottoolbar')
-        self.connect(self.toolbar, SIGNAL('popoutRequested'),
-                     self.on_actionPopOut_triggered)
+        self.toolbar.popoutRequested.connect(self.on_actionPopOut_triggered)
         self.addToolBar(self.toolbar)
         layout2.addWidget(self.canvas)
         self.plotframe.setLayout(layout2)
 
         # session events
-        self.connect(session, SIGNAL('itemsUpdated'),
-                     self.on_session_itemsUpdated)
-        self.connect(session, SIGNAL('itemAdded'), self.on_session_itemAdded)
-        self.connect(session, SIGNAL('filenameChanged'),
-                     self.on_session_filenameChanged)
-        self.connect(session, SIGNAL('dirtyChanged'),
-                     self.on_session_dirtyChanged)
+        session.itemsUpdated.connect(self.on_session_itemsUpdated)
+        session.itemAdded.connect(self.on_session_itemAdded)
+        session.filenameChanged.connect(self.on_session_filenameChanged)
+        session.dirtyChanged.connect(self.on_session_dirtyChanged)
 
         # create data loader
         self.dloader = DataLoader(self, self.canvas.plotter)
-        self.connect(self.dloader, SIGNAL('newDatas'), self.on_dloader_newDatas)
+        self.dloader.newDatas.connect(self.on_dloader_newDatas)
         self.stacker.addWidget(self.dloader)
         self.current_panel = self.dloader
 
@@ -97,8 +98,8 @@ class UFitMain(QMainWindow):
         self.itemTree.addAction(self.actionMergeData)
         self.itemTree.addAction(self.actionRemoveData)
         self.itemTree.addAction(self.menuMoveToGroup.menuAction())
-        self.connect(self.itemTree, SIGNAL('newSelection'),
-                     self.on_itemTree_newSelection)
+        self.itemTree.addAction(self.menuCopyToGroup.menuAction())
+        self.itemTree.newSelection.connect(self.on_itemTree_newSelection)
 
         # backend selector
         self.backend_group = QActionGroup(self)
@@ -109,14 +110,14 @@ class UFitMain(QMainWindow):
                 action.setChecked(True)
             self.backend_group.addAction(action)
             self.menuBackend.addAction(action)
-            self.connect(action, SIGNAL('triggered()'),
-                         self.on_backend_action_triggered)
+            action.triggered.connect(self.on_backend_action_triggered)
 
         # manage button
         menu = QMenu(self)
         menu.addAction(self.actionMergeData)
         menu.addAction(self.actionRemoveData)
         menu.addMenu(self.menuMoveToGroup)
+        menu.addMenu(self.menuCopyToGroup)
         menu.addAction(self.actionReorder)
         menu.addSeparator()
         menu.addAction(self.actionNewGroup)
@@ -166,7 +167,7 @@ class UFitMain(QMainWindow):
 
     def on_session_itemsUpdated(self):
         # remove all panels whose item has vanished
-        for item, panel in self.itempanels.items():
+        for item, panel in listitems(self.itempanels):
             if item not in session.all_items:
                 self.stacker.removeWidget(panel)
                 del self.itempanels[item]
@@ -205,38 +206,39 @@ class UFitMain(QMainWindow):
         if recent_files:
             for i, fname in enumerate(recent_files):
                 action = QAction('%d - %s' % (i+1, fname), self)
-                self.connect(action, SIGNAL("triggered()"), self.load_session)
+                action.triggered.connect(self.load_session)
                 action.setData(fname)
                 self.menuRecent.addAction(action)
         self.actionClearRecent.setEnabled(len(recent_files) > 0)
         self.menuRecent.addSeparator()
         self.menuRecent.addAction(self.actionClearRecent)
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionClearRecent_triggered(self):
         """Clear recent files list"""
         self.recent_files = []
 
     def select_new_panel(self, panel):
-        if hasattr(self.current_panel, 'save_limits'):
-            self.current_panel.save_limits()
-        self.current_panel = panel
-        self.stacker.setCurrentWidget(self.current_panel)
+        if panel is not self.current_panel:
+            if hasattr(self.current_panel, 'save_limits'):
+                self.current_panel.save_limits()
+            self.current_panel = panel
+        self.stacker.setCurrentWidget(self.current_panel)  # doesn't hurt
 
     def on_canvas_pick(self, event):
         if isinstance(self.current_panel, ScanDataPanel):
             self.current_panel.on_canvas_pick(event)
 
     def on_canvas_click(self, event):
-        if event.button == 3 and not self.toolbar.mode: # right button
+        if event.button == 3 and not self.toolbar.mode:  # right button
             self.canvasMenu.popup(QCursor.pos())
 
-    @qtsig('')
+    @pyqtSlot()
     def on_loadBtn_clicked(self):
         self.select_new_panel(self.dloader)
         self.itemTree.setCurrentIndex(QModelIndex())
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionNewGroup_triggered(self):
         name = QInputDialog.getText(self, 'ufit', 'Please enter a name '
                                     'for the new group:')[0]
@@ -248,51 +250,68 @@ class UFitMain(QMainWindow):
         self.menuMoveToGroup.clear()
         for group in session.groups:
             action = QAction(group.name, self)
-            def move_to(group=group):
+
+            def move_to(_arg=None, group=group):
                 items = self.selected_items()
                 if not items:
                     return
                 session.move_items(items, group)
                 self.re_expand_tree()
-            self.connect(action, SIGNAL('triggered()'), move_to)
+            action.triggered.connect(move_to)
             self.menuMoveToGroup.addAction(action)
 
+    def on_menuCopyToGroup_aboutToShow(self):
+        self.menuCopyToGroup.clear()
+        for group in session.groups:
+            action = QAction(group.name, self)
+
+            def copy_to(_arg=None, group=group):
+                items = self.selected_items()
+                if not items:
+                    return
+                session.copy_items(items, group)
+                self.re_expand_tree()
+            action.triggered.connect(copy_to)
+            self.menuCopyToGroup.addAction(action)
+
     def on_menuRemoveGroup_aboutToShow(self):
         self.menuRemoveGroup.clear()
         for group in session.groups:
             action = QAction(group.name, self)
-            def remove(group=group):
+
+            def remove(_arg=None, group=group):
                 session.remove_group(group)
                 self.re_expand_tree()
-            self.connect(action, SIGNAL('triggered()'), remove)
+            action.triggered.connect(remove)
             self.menuRemoveGroup.addAction(action)
 
     def on_menuRenameGroup_aboutToShow(self):
         self.menuRenameGroup.clear()
         for group in session.groups:
             action = QAction(group.name, self)
-            def rename(group=group):
+
+            def rename(_arg=None, group=group):
                 name = QInputDialog.getText(self, 'ufit', 'Please enter a new name '
                                             'for the group:', text=group.name)[0]
                 if not name:
                     return
                 session.rename_group(group, name)
-            self.connect(action, SIGNAL('triggered()'), rename)
+            action.triggered.connect(rename)
             self.menuRenameGroup.addAction(action)
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionRemoveData_triggered(self):
         items = self.selected_items()
         if not items:
             return
         if QMessageBox.question(self, 'ufit',
                                 'OK to remove %d item(s)?' % len(items),
-                                QMessageBox.Yes|QMessageBox.No) == QMessageBox.No:
+                                QMessageBox.Yes | QMessageBox.No) == QMessageBox.No:
             return
         session.remove_items(items)
         self.re_expand_tree()
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionReorder_triggered(self):
         dlg = QDialog(self)
         loadUi(dlg, 'reorder.ui')
@@ -335,7 +354,7 @@ class UFitMain(QMainWindow):
             item = items[0]
             if item not in self.itempanels:
                 panel = self.itempanels[item] = \
-                        item.create_panel(self, self.canvas)
+                    item.create_panel(self, self.canvas)
                 self.stacker.addWidget(panel)
             else:
                 panel = self.itempanels[item]
@@ -346,9 +365,11 @@ class UFitMain(QMainWindow):
                 self.inspector_window.setDataset(item.data)
         else:
             paneltype = type(items[0])
+            if not hasattr(paneltype, 'create_multi_panel'):
+                return
             if paneltype not in self.multipanels:
                 panel = self.multipanels[paneltype] = \
-                        items[0].create_multi_panel(self, self.canvas)
+                    items[0].create_multi_panel(self, self.canvas)
                 self.stacker.addWidget(panel)
             else:
                 panel = self.multipanels[paneltype]
@@ -372,33 +393,35 @@ class UFitMain(QMainWindow):
             else:
                 self.itemTree.collapse(self.itemlistmodel.index_for_group(group))
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionInspector_triggered(self):
         if self.inspector_window:
             self.inspector_window.activateWindow()
             return
         self.inspector_window = InspectorWindow(self)
+
         def deref():
             self.inspector_window = None
-        self.connect(self.inspector_window, SIGNAL('replotRequest'),
-                     lambda: self.current_panel.plot(True))
-        self.connect(self.inspector_window, SIGNAL('closed'), deref)
+        self.inspector_window.replotRequest.connect(
+            lambda: self.current_panel.plot(True))
+        self.inspector_window.closed.connect(deref)
         if isinstance(self.current_panel, ScanDataPanel):
             self.inspector_window.setDataset(self.current_panel.item.data)
         self.inspector_window.show()
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionAnnotations_triggered(self):
         if self.annotation_window:
             self.annotation_window.activateWindow()
             return
         self.annotation_window = AnnotationWindow(self)
+
         def deref():
             self.annotation_window = None
-        self.connect(self.annotation_window, SIGNAL('closed'), deref)
+        self.annotation_window.closed.connect(deref)
         self.annotation_window.show()
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionLoadData_triggered(self):
         self.on_loadBtn_clicked()
 
@@ -428,7 +451,7 @@ class UFitMain(QMainWindow):
 
     def _get_export_filename(self, filter='ASCII text (*.txt)'):
         initialdir = session.props.get('lastexportdir', session.dirname)
-        filename = QFileDialog.getSaveFileName(
+        filename, _ = QFileDialog.getSaveFileName(
             self, 'Select export file name', initialdir, filter)
         if filename == '':
             return ''
@@ -436,25 +459,25 @@ class UFitMain(QMainWindow):
         session.props.lastexportdir = path.dirname(expfilename)
         return expfilename
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionExportASCII_triggered(self):
         expfilename = self._get_export_filename()
         if expfilename:
             self.current_panel.export_ascii(expfilename)
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionExportFIT_triggered(self):
         expfilename = self._get_export_filename()
         if expfilename:
             self.current_panel.export_fits(expfilename)
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionExportPython_triggered(self):
         expfilename = self._get_export_filename('Python files (*.py)')
         if expfilename:
             self.current_panel.export_python(expfilename)
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionExportParams_triggered(self):
         items = self.selected_items(ScanDataItem)
         dlg = ParamExportDialog(self, items)
@@ -464,16 +487,16 @@ class UFitMain(QMainWindow):
         if expfilename:
             try:
                 dlg.do_export(expfilename)
-            except Exception, e:
+            except Exception as e:
                 logger.exception('While exporting parameters')
                 QMessageBox.warning(self, 'Error', 'Could not export '
                                     'parameters: %s' % e)
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionPrint_triggered(self):
         self.canvas.print_()
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionPopOut_triggered(self):
         new_win = QMainWindow()
         canvas = MPLCanvas(new_win)
@@ -493,17 +516,17 @@ class UFitMain(QMainWindow):
             iw.executeCommand('from ufit.lab import *')
             iw.executeCommand('sca(ax)')
             iw.setFocus()
-            self.connect(iw, SIGNAL('redraw'), canvas.draw_idle)
+            iw.redrawme.connect(canvas.draw_idle)
             splitter.addWidget(iw)
         new_win.setCentralWidget(splitter)
         self.current_panel.plot(limits=None, canvas=canvas)
         new_win.show()
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionSavePlot_triggered(self):
-        self.canvas.save_figure()
+        self.toolbar.save_figure()
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionUnzoom_triggered(self):
         self.toolbar.home()
 
@@ -511,22 +534,22 @@ class UFitMain(QMainWindow):
         if not self.isWindowModified():  # nothing there to be saved
             return True
         resp = QMessageBox.question(self, 'ufit',
-            'Save current session?\n%s' % session.filename,
-            QMessageBox.Yes|QMessageBox.No|QMessageBox.Cancel)
+                                    'Save current session?\n%s' % session.filename,
+                                    QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
         if resp == QMessageBox.Yes:
             return self.save_session()
         elif resp == QMessageBox.No:
             return True
         return False
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionNewSession_triggered(self):
         if not self.check_save():
             return
         session.clear()
         self.on_loadBtn_clicked()
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionLoad_triggered(self):
         if not self.check_save():
             return
@@ -534,19 +557,19 @@ class UFitMain(QMainWindow):
         if not initialdir:
             with self.sgroup as settings:
                 initialdir = settings.value('loadfiledirectory', '')
-        filename = QFileDialog.getOpenFileName(
+        filename, _ = QFileDialog.getOpenFileName(
             self, 'Select file name', initialdir, 'ufit files (*.ufit)')
         if filename == '':
             return
         self.load_session(path_to_str(filename))
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionInsert_triggered(self):
         initialdir = session.dirname
         if not initialdir:
             with self.sgroup as settings:
                 initialdir = settings.value('loadfiledirectory', '')
-        filename = QFileDialog.getOpenFileName(
+        filename, _ = QFileDialog.getOpenFileName(
             self, 'Select file name', initialdir, 'ufit files (*.ufit)')
         if filename == '':
             return
@@ -557,16 +580,19 @@ class UFitMain(QMainWindow):
             # Recent files action
             action = self.sender()
             if isinstance(action, QAction):
+                if not self.check_save():
+                    return
                 filename = action.data()
         try:
             session.load(filename)
             with self.sgroup as settings:
                 settings.setValue('loadfiledirectory', path.dirname(filename))
-        except Exception, err:
+        except Exception as err:
             logger.exception('Loading session %r failed' % filename)
             QMessageBox.warning(self, 'Error', 'Loading failed: %s' % err)
         else:
             self.re_expand_tree()
+            self.setWindowModified(False)
             # if there are annotations, show the window automatically
             if session.props.get('annotations'):
                 self.on_actionAnnotations_triggered()
@@ -582,20 +608,24 @@ class UFitMain(QMainWindow):
             for item in group.items:
                 session.add_item(item)
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionSave_triggered(self):
         self.save_session()
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionSaveAs_triggered(self):
         self.save_session_as()
 
+    @pyqtSlot()
+    def on_actionQExplorer_triggered(self):
+        ReciprocalViewer(self)
+
     def save_session(self):
         if session.filename is None:
             return self.save_session_as()
         try:
             session.save()
-        except Exception, err:
+        except Exception as err:
             logger.exception('Saving session failed')
             QMessageBox.warning(self, 'Error', 'Saving failed: %s' % err)
             return False
@@ -603,20 +633,20 @@ class UFitMain(QMainWindow):
 
     def save_session_as(self):
         initialdir = session.dirname
-        filename = QFileDialog.getSaveFileName(
+        filename, _ = QFileDialog.getSaveFileName(
             self, 'Select file name', initialdir, 'ufit files (*.ufit)')
         if filename == '':
             return False
         session.set_filename(path_to_str(filename))
         try:
             session.save()
-        except Exception, err:
+        except Exception as err:
             logger.exception('Saving session failed')
             QMessageBox.warning(self, 'Error', 'Saving failed: %s' % err)
             return False
         return True
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionMergeData_triggered(self):
         items = self.selected_items(ScanDataItem)
         if len(items) < 2:
@@ -632,7 +662,7 @@ class UFitMain(QMainWindow):
             new_data = datalist[0].merge(precision, *datalist[1:])
             session.add_item(ScanDataItem(new_data), self.items[-1].group)
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionQuit_triggered(self):
         self.close()
 
@@ -648,7 +678,7 @@ class UFitMain(QMainWindow):
             settings.setValue('vsplitstate', self.vsplitter.saveState())
             settings.setValue('recentfiles', self.recent_files)
 
-    @qtsig('')
+    @pyqtSlot()
     def on_actionAbout_triggered(self):
         dlg = QDialog(self)
         dlg.setWindowFlags(Qt.Dialog | Qt.FramelessWindowHint)
@@ -656,6 +686,6 @@ class UFitMain(QMainWindow):
         dlg.lbl.setText(dlg.lbl.text().replace('VERSION', __version__))
         dlg.exec_()
 
-    @qtsig('')
+    @pyqtSlot()
     def on_backend_action_triggered(self):
         backends.set_backend(str(self.sender().text()))
