@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -65,7 +65,8 @@ class GaussInt(Model):
         self.params[2].finalize = abs
 
         self.fcn = lambda p, x: \
-            abs(p[pint]) / (abs(p[pf]) * sqrt(2*pi)) * exp(-(x - p[pp])**2/p[pf]**2 * 4*log(2))
+            abs(p[pint]) / (abs(p[pf]) * sqrt(pi/(4 * log(2)))) * \
+            exp(-(x - p[pp])**2/p[pf]**2 * 4*log(2))
 
     pick_points = ['peak', 'width']
 
@@ -274,10 +275,9 @@ class Gauss2D(Model):
         def fcn(p, x):
             # rotate coordinate system by theta
             c, s = cos(p[pth]), sin(p[pth])
-            x1 = (x[:,0] - p[ppx])*c - (x[:,1] - p[ppy])*s
-            y1 = (x[:,0] - p[ppx])*s + (x[:,1] - p[ppy])*c
+            x1 = (x[:, 0] - p[ppx])*c - (x[:, 1] - p[ppy])*s
+            y1 = (x[:, 0] - p[ppx])*s + (x[:, 1] - p[ppy])*c
             return abs(p[pb]) + abs(p[pa]) * \
                 exp(-x1**2/p[pfx]**2 * 4*log(2)) * \
                 exp(-y1**2/p[pfy]**2 * 4*log(2))
         self.fcn = fcn
-
