@@ -2,13 +2,12 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
 """Model of S(q,w) with resolution convolution for TAS."""
 
-import time
 import inspect
 
 from numpy import matrix as zeros
@@ -17,6 +16,7 @@ from ufit.rescalc import resmat, calc_MC, calc_MC_cluster, calc_MC_mcstas, \
     load_cfg, load_par, PARNAMES, CFGNAMES, plot_resatpoint
 from ufit.models.base import Model
 from ufit.param import prepare_params, update_params
+from ufit.pycompat import string_types
 
 __all__ = ['ConvolvedScatteringLaw']
 
@@ -53,7 +53,7 @@ class ConvolvedScatteringLaw(Model):
                  mcstas=None, matrix=None, mathkl=None, **init):
         self._cluster = False
         self._mcstas = mcstas
-        if isinstance(sqw, str):
+        if isinstance(sqw, string_types):
             modname, funcname = sqw.split(':')
             mod = __import__(modname)
             code = open(mod.__file__.rstrip('c')).read()
@@ -99,14 +99,14 @@ class ConvolvedScatteringLaw(Model):
 
         if par_all:
             instparnames = [pn for pn in instparnames if not pn.startswith('par_')]
-            self._instpars = [pn for pn in self._instpars if not isinstance(pn, str)]
+            self._instpars = [pn for pn in self._instpars if not isinstance(pn, string_types)]
             instparnames.extend('par_' + pn for pn in PARNAMES)
             self._instpars.extend(PARNAMES)
             for pn in PARNAMES:
                 init['par_' + pn] = str(par_orig[pn])
         if cfg_all:
             instparnames = [pn for pn in instparnames if not pn.startswith('cfg_')]
-            self._instpars = [ip for ip in self._instpars if isinstance(ip, str)]
+            self._instpars = [ip for ip in self._instpars if isinstance(ip, string_types)]
             instparnames.extend('cfg_' + x for x in CFGNAMES)
             self._instpars.extend(range(len(CFGNAMES)))
             for i in range(len(CFGNAMES)):
@@ -115,25 +115,26 @@ class ConvolvedScatteringLaw(Model):
         # numba compat
         arg_sqw = getattr(self._sqw, 'py_func', self._sqw)
         self._pvs = self._init_params(name,
-            ['NMC', 'bkgd'] + inspect.getargspec(arg_sqw)[0][6:] +
-            instparnames, init)
+                                      ['NMC', 'bkgd'] +
+                                      inspect.getargspec(arg_sqw)[0][6:] +
+                                      instparnames, init)
         self._ninstpar = len(instparnames)
         self._resmat = resmat(cfg_orig, par_orig)
 
         if matrix is not None:
             self._resmat.fixed_res = True
             self._resmat.setNPMatrix(matrix, mathkl)
-            #self._resmat.NP = matrix
+            # self._resmat.NP = matrix
             self._resmat.R0_corrected = 1.0
 
     def fcn(self, p, x):
         parvalues = [p[pv] for pv in self._pvs]
-        t1 = time.time()
-        #print 'Sqw: values = ', parvalues
+        # t1 = time.time()
+        # print 'Sqw: values = ', parvalues
         if self._ninstpar:
             sqwpar  = parvalues[2:-self._ninstpar]
             for pn, pv in zip(self._instpars, parvalues[-self._ninstpar:]):
-                if isinstance(pn, str):
+                if isinstance(pn, string_types):
                     self._resmat.par[pn] = pv
                 else:
                     self._resmat.cfg[pn] = pv
@@ -152,8 +153,8 @@ class ConvolvedScatteringLaw(Model):
             res = calc_MC(x, sqwpar, self._sqw, self._resmat,
                           parvalues[0], use_caching=use_caching)
         res += parvalues[1]  # background
-        t2 = time.time()
-        #print 'Sqw: iteration = %.3f sec' % (t2-t1)
+        # t2 = time.time()
+        # print 'Sqw: iteration = %.3f sec' % (t2-t1)
         return res
 
     def resplot(self, h, k, l, e):
