@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -11,11 +11,13 @@
 import re
 import copy
 import numpy as np
+import scipy
 
 from ufit import UFitError
+from ufit.pycompat import iteritems, listitems, number_types, string_types
 
-__all__ = ['fixed', 'expr', 'overall', 'datapar', 'limited', 'delta',
-           'Param', 'expr_namespace']
+__all__ = ['fixed', 'expr', 'overall', 'datapar', 'datainit', 'limited',
+           'delta', 'Param', 'expr_namespace']
 
 
 class fixed(str):
@@ -24,8 +26,7 @@ class fixed(str):
     The ``fixed()`` wrapper can also be omitted; any string means that a
     parameter as a fixed value that can include references to other parameters.
     """
-    def __init__(self, p):
-        str.__init__(self, p)
+
 
 class expr(str):
     """Mark the parameter value as an expression.
@@ -33,8 +34,7 @@ class expr(str):
     The ``expr()`` wrapper can also be omitted; any string means that a
     parameter as a fixed value that can include references to other parameters.
     """
-    def __init__(self, e):
-        str.__init__(self, e)
+
 
 class overall(object):
     """Mark the parameter as an "overall" (global) parameter in a global fit.
@@ -45,6 +45,7 @@ class overall(object):
     def __init__(self, v):
         self.v = v
 
+
 class datapar(object):
     """Mark the parameter as coming from the data file's metadata.
 
@@ -53,6 +54,15 @@ class datapar(object):
     def __init__(self, v):
         self.v = v
 
+
+class datainit(object):
+    """Mark the parameter as initially coming from the data file, but free
+    for fitting.
+    """
+    def __init__(self, v):
+        self.v = v
+
+
 class limited(tuple):
     """Give parameter limits together with the initial value.
 
@@ -63,6 +73,7 @@ class limited(tuple):
     def __new__(self, min, max, v):
         return (min, max, v)
 
+
 class delta(object):
     """Give parameter delta together with initial value."""
     def __init__(self, delta, v):
@@ -73,6 +84,8 @@ class delta(object):
 expr_namespace = {
     'data': None,  # replaced by the dataset's metadata dict, but in here
                    # so that no parameter can be called "data"
+    'numpy': np,
+    'scipy': scipy,
 }
 for fcn in ['pi', 'sqrt', 'sin', 'cos', 'tan', 'arcsin', 'arccos',
             'arctan', 'exp', 'log', 'radians', 'degrees', 'ceil',
@@ -84,7 +97,7 @@ id_re = re.compile('[a-zA-Z_][a-zA-Z0-9_]*$')
 
 class Param(object):
     def __init__(self, name, value=0, expr=None, pmin=None, pmax=None,
-                 overall=False, delta=0, error=0, correl=None,
+                 overall=False, delta=0, error=0, correl=None, initexpr=None,
                  finalize=lambda x: x):
         if not id_re.match(name):
             raise UFitError('Parameter name %r is not a valid Python '
@@ -94,6 +107,7 @@ class Param(object):
         self.name = name
         self.value = value
         self.expr = expr
+        self.initexpr = initexpr
         self.pmin = pmin
         self.pmax = pmax
         # true if a global parameter for a global fit
@@ -111,13 +125,16 @@ class Param(object):
         if isinstance(pdef, cls):
             return pdef
         self = cls(name)
-        while not isinstance(pdef, (int, long, float, str)):
+        while not isinstance(pdef, (number_types, string_types)):
             if isinstance(pdef, overall):
                 self.overall = True
                 pdef = pdef.v
             elif isinstance(pdef, datapar):
                 self.expr = 'data.' + pdef.v
                 pdef = 0
+            elif isinstance(pdef, datainit):
+                self.initexpr = 'data.' + pdef.v
+                pdef = 0
             elif isinstance(pdef, delta):
                 self.delta = pdef.delta
                 pdef = pdef.v
@@ -126,7 +143,7 @@ class Param(object):
             else:
                 raise UFitError('Parameter definition %s not understood' %
                                 pdef)
-        if isinstance(pdef, str):
+        if isinstance(pdef, string_types):
             self.expr = pdef
         else:
             self.value = pdef
@@ -147,10 +164,11 @@ class Param(object):
     def __reduce__(self):
         return (Param, (self.name, self.value, self.expr, self.pmin,
                         self.pmax, self.overall, self.delta, self.error,
-                        self.correl))
+                        self.correl, self.initexpr))
 
     def __str__(self):
-        s = '%-15s = %10.5g +/- %10.5g' % (self.name, self.value, self.error)
+        error = float('inf') if self.error is None else self.error
+        s = '%-15s = %10.5g +/- %10.5g' % (self.name, self.value, error)
         if self.expr:
             s += ' (fixed: %s)' % self.expr
         if self.overall:
@@ -176,12 +194,18 @@ class Param(object):
 # XXX replace by something more safe later
 param_eval = eval
 
+
 def prepare_params(params, meta):
     # find parameters that need to vary
     dependent = {}
     varying = []
     varynames = []
     for p in params:
+        if p.initexpr:
+            try:
+                p.value = param_eval(p.initexpr, {'data': meta})
+            except Exception:
+                pass  # can happen for heterogeneous data collections
         if p.expr:
             dependent[p.name] = [p.expr, None]
         else:
@@ -199,19 +223,19 @@ def prepare_params(params, meta):
         maxit -= 1
         if maxit == 0:
             s = '\n'.join('   %s: %s' % (k, v[1]) for (k, v)
-                          in dependent.iteritems())
+                          in iteritems(dependent))
             raise UFitError('Detected unresolved parameter dependencies:\n' + s)
-        for p, (expr, _) in dependent.items():  # dictionary will change
+        for p, (expr, _) in listitems(dependent):  # dictionary will change
             try:
                 pd[p] = param_eval(expr, pd)
-            except NameError, e:
+            except NameError as e:
                 dependent[p][1] = str(e)
-            except AttributeError, e:
+            except AttributeError as e:
                 dependent[p][1] = 'depends on data.' + str(e)
             else:
                 del dependent[p]
                 dep_order.append((p, expr))
-    #pd.pop('__builtins__', None)
+    # pd.pop('__builtins__', None)
 
     return varying, varynames, dep_order, pd
 
@@ -221,4 +245,4 @@ def update_params(parexprs, meta, pd):
     pd['data'] = meta
     for p, expr in parexprs:
         pd[p] = param_eval(expr, pd)
-    #pd.pop('__builtins__', None)
+    # pd.pop('__builtins__', None)
