@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -15,6 +15,7 @@ from matplotlib import pyplot as pl
 
 from ufit.utils import cached_property
 from ufit.plotting import DataPlotter
+from ufit.pycompat import iteritems
 
 __all__ = ['Result']
 
@@ -103,16 +104,16 @@ class Result(object):
            chi^2/NDF       =      1.491
            =====================================================================
         """
-        print 'Fit results for %s' % self.data.name
+        print('Fit results for %s' % self.data.name)
         if not self.success:
-            print 'FIT FAILED: ' + self.message
+            print('FIT FAILED: ' + self.message)
         elif self.message:
-            print '>', self.message
-        print '-' * 80
+            print('> %s' % self.message)
+        print('-' * 80)
         for p in self.params:
-            print p
-        print '%-15s = %10.4g' % ('chi^2/NDF', self.chisqr)
-        print '=' * 80
+            print(p)
+        print('%-15s = %10.4g' % ('chi^2/NDF', self.chisqr))
+        print('=' * 80)
 
     def plot(self, axes=None, params=True, multi=False):
         """Plot the data and model together in the current figure.
@@ -141,10 +142,10 @@ class Result(object):
 
 
 def calc_panel_size(num):
-    for nx, ny in ([1,1], [2,1], [2,2], [3,2], [3,3], [4,3], [5,3], [4,4],
-                   [5,4], [6,4], [5,5], [6,5], [7,5], [6,6], [8,5], [7,6],
-                   [9,5], [8,6], [7,7], [9,6], [8,7], [9,7], [8,8], [10,7],
-                   [9,8], [11,7], [9,9], [11,8], [10,9], [12,8], [11,9], [10,10]):
+    for nx, ny in ([1, 1], [2, 1], [2, 2], [3, 2], [3, 3], [4, 3], [5, 3], [4, 4],
+                   [5, 4], [6, 4], [5, 5], [6, 5], [7, 5], [6, 6], [8, 5], [7, 6],
+                   [9, 5], [8, 6], [7, 7], [9, 6], [8, 7], [9, 7], [8, 8], [10, 7],
+                   [9, 8], [11, 7], [9, 9], [11, 8], [10, 9], [12, 8], [11, 9], [10, 10]):
         if nx*ny >= num:
             return nx, ny
     return num//10 + 1, 10
@@ -156,13 +157,13 @@ class MultiResult(list):
         fig, axarray = pl.subplots(dims[1], dims[0])
         for res, axes in zip(self, ravel(axarray)):
             res.plotfull(axes=axes)
-        #pl.tight_layout()
+        # pl.tight_layout()
 
     @cached_property
     def datavalues(self):
-        d = dict((k, [v]) for (k, v) in self[0].data.meta.iteritems())
+        d = dict((k, [v]) for (k, v) in iteritems(self[0].data.meta))
         for res in self[1:]:
-            for k, v in res.data.meta.iteritems():
+            for k, v in iteritems(res.data.meta):
                 d[k].append(v)
         return d
 
@@ -194,12 +195,12 @@ class MultiResult(list):
 
     def printout(self):
         """Print global parameters of the fit."""
-        print 'OVERALL fit results'
-        print '-' * 80
+        print('OVERALL fit results')
+        print('-' * 80)
         for p in self[0].params:
             if p.overall:
-                print p
-        print '=' * 80
+                print(p)
+        print('=' * 80)
 
     def plot_param(self, xname, pname):
         pl.errorbar(self.datavalues[xname], self.paramvalues[pname],
