@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
+# Copyright (c) 2013-2019, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -25,8 +25,10 @@ class attrdict(dict):
             return self[key]
         except KeyError:
             raise AttributeError(key)
+
     def __setattr__(self, key, value):
         self[key] = value
+
     def __delattr__(self, key):
         try:
             del self[key]
@@ -36,6 +38,7 @@ class attrdict(dict):
 
 _missing = object()
 
+
 class cached_property(object):
     """A decorator that converts a function into a lazy property.  The
     function wrapped is called the first time to retrieve the result
@@ -61,15 +64,23 @@ class cached_property(object):
 
 numor_re = re.compile(r'\d+')
 
+
 def extract_template(fn):
     bn = path.basename(fn)
     dn = path.dirname(fn)
     m = list(numor_re.finditer(bn))
+    last_dot = bn.rfind('.')
     if not m:
         dtempl = fn
         numor = 0
     else:
-        b, e = m[-1].span()
+        ix = -1
+        b, e = m[ix].span()
+        # ignore numbers if they are part of the extension *and* there is
+        # other numbers
+        if b > last_dot and len(m) > 1:
+            ix = -2
+            b, e = m[ix].span()
         dtempl = path.join(dn, bn[:b] + '%%0%dd' % (e-b) + bn[e:])
-        numor = int(m[-1].group())
+        numor = int(m[ix].group())
     return dtempl, numor
